#pragma once
#include "Deer/DataStore.h"
#include "Deer/Memory.h"
#include "Deer/Path.h"

#ifdef DEER_RENDER
#include "DeerRender/GizmoRenderer.h"
#endif

#include <string>
#include <vector>

namespace Deer {
	class VoxelWorld;
	class VoxelWorldProps;
	class Environment;

	class Scene {
	   public:
		Scene();
		~Scene();
		Scene(const Scene&) = delete;
		Scene operator=(Scene&) = delete;

		void createVoxelWorld(const VoxelWorldProps&);
		void deleteVoxelWorld();
		void clear();

		void beginExecution();
		void updateInternalVars();
		void endExecution();

	   public:
		inline Environment& getMainEnviroment() { return *m_enviroment; }
		inline VoxelWorld& getVoxelWorld() { return *m_voxelWorld; }
		inline bool isVoxelWorldInitialized() {
			return m_voxelWorld != nullptr;
		}
		inline bool getExecutingState() { return m_isExecuting; }

	   private:
		Scope<Environment> m_enviroment;
		Scope<VoxelWorld> m_voxelWorld;

		bool m_isExecuting = false;
#ifdef DEER_RENDER
	   public:
		void render();
		void render(SceneCamera);
		inline GizmoRenderer& getMainGizmoRenderer() { return m_gizmoRenderer; }

	   private:
		GizmoRenderer m_gizmoRenderer;
#endif
	};

	namespace SceneDataStore {
		void loadScene(Scene& scene, const Path& name);
		void exportScene(const Scene& scene, const Path& name);

		void exportRuntimeScene(const Scene& scene);
		void importRuntimeScene(Scene& scene);
	}  // namespace SceneDataStore
}  // namespace Deer
